using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Input;

namespace ErrorReporting
{
    public class ExceptionGame : Game
    {
        //The gamerTag and email constants need personalized for each developer/game
        private const string gamerTag = "XNADevelopment";
        private const string email = "clingermangw@hotmail.com";
                
        private GameInput input;
        private const string ActionExit = "Exit";
        private const string ActionSendMessage = "SendMessage";

        private readonly Exception exception;
        private SpriteBatch batch;
        private SpriteFont font;

        public ExceptionGame(Exception e)
        {
            //Set the resolution to the recommended one
            //for the Xbox360
            new GraphicsDeviceManager(this)
            {
                PreferredBackBufferWidth = 1280,
                PreferredBackBufferHeight = 720
            };
            exception = e;

            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            if (!GamerServicesDispatcher.IsInitialized)
            {
                GamerServicesDispatcher.Initialize(Services);
            }
            GamerServicesDispatcher.WindowHandle = Window.Handle;
            base.Initialize();
        }

        protected override void LoadContent()
        {
            batch = new SpriteBatch(GraphicsDevice);
            font = Content.Load<SpriteFont>("Debug");

            //Setup input for the game
            input = new GameInput();
            input.AddGamePadInput(ActionExit, Buttons.B, true);
            input.AddKeyboardInput(ActionExit, Keys.B, true);
            input.AddGamePadInput(ActionSendMessage, Buttons.A, true);
            input.AddKeyboardInput(ActionSendMessage, Keys.A, true);
        }

        protected override void Update(GameTime gameTime)
        {
            GamerServicesDispatcher.Update();

            input.BeginUpdate();

            if (input.IsPressed(ActionExit, null))
            {
                Exit();
            }

#if XBOX
            PlayerIndex aControllingPlayer;
            if (input.IsPressed(ActionSendMessage, null, out aControllingPlayer))
            {
                SendMessage(aControllingPlayer);
            }
#endif

            input.EndUpdate();
            base.Update(gameTime);
        }

        private void SendMessage(PlayerIndex theControllingPlayer)
        {
            if (!Guide.IsVisible && IsSendMessageAllowed(theControllingPlayer))
            {
                string message = "Error Report: " + exception.ToString();
                Guide.ShowComposeMessage(theControllingPlayer, message.Substring(0, 255), null);
            }
        }

        bool? isSendMessageAllowed = null;
        PlayerIndex? controllingPlayer = null;
        private bool IsSendMessageAllowed(PlayerIndex theControllingPlayer)
        {
#if XBOX
            if (isSendMessageAllowed == null || theControllingPlayer != controllingPlayer)
            {
                controllingPlayer = theControllingPlayer;
                SignedInGamer gamer = Gamer.SignedInGamers[theControllingPlayer];
                if (gamer == null || !gamer.IsSignedInToLive || gamer.Privileges.AllowCommunication != GamerPrivilegeSetting.Everyone)
                {
                    isSendMessageAllowed = false;
                }
                else
                {
                    isSendMessageAllowed = true;
                }
            }
#endif

#if WINDOWS
            isSendMessageAllowed = false;
#endif

            return (bool)isSendMessageAllowed;
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            Vector2 position = new Vector2(
                   GraphicsDevice.Viewport.TitleSafeArea.X,
                   GraphicsDevice.Viewport.TitleSafeArea.Y);
            batch.Begin();

            batch.DrawString(font, "The game encountered an unexpected error and had " + 
                System.Environment.NewLine + "to shut down. " +
                "We are sorry for the inconvenience.", position, Color.White);

#if XBOX
            if (isSendMessageAllowed == null || isSendMessageAllowed == true)
            {
                batch.DrawString(font, "Press A to send the error message to the developer. " +
                    System.Environment.NewLine + "You must choose to manually enter the gamertag " + 
                    System.Environment.NewLine + gamerTag + " to send the message to.", position + new Vector2(0, 80), Color.White);
            }
            else
            {
                batch.DrawString(font, "Profile unable to send messages. Thanks for trying!", position + new Vector2(0, 80), Color.Blue);
            }
#endif

            batch.DrawString(font, "You can send information about this error message " + System.Environment.NewLine +
                "to " + email, position + new Vector2(0, 195), Color.White);
            batch.DrawString(font, "Press B to Exit", position + new Vector2(0, 280), Color.White);
            batch.DrawString(font, exception.ToString(), position + new Vector2(0, 325), Color.White);

            batch.End();

            base.Draw(gameTime);
        }
    }
}