﻿using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace ErrorReporting
{
    public class GameInput
    {
        Dictionary<string, Input> mInputs = new Dictionary<string, Input>();

        public GameInput()
        {
        }

        public Input MyInput(string theAction)
        {
            //Add the Action if it doesn't already exist
            if (mInputs.ContainsKey(theAction) == false)
            {
                mInputs.Add(theAction, new Input());
            }

            return mInputs[theAction];
        }

        public void BeginUpdate()
        {
            Input.BeginUpdate();
        }

        public void EndUpdate()
        {
            Input.EndUpdate();
        }

        public bool IsConnected(PlayerIndex thePlayer)
        {
            //If there never WAS a gamepad connected, then we'll just
            //say that the gamepad is still connected...
            if (Input.GamepadConnectionState[thePlayer] == false)
            {
                return true;
            }

            return Input.IsConnected(thePlayer);
        }

        public bool IsPressed(string theAction, PlayerIndex thePlayer)
        {
            if (mInputs.ContainsKey(theAction) == false)
            {
                return false;
            }

            return mInputs[theAction].IsPressed(thePlayer);
        }

        public bool IsPressed(string theAction, PlayerIndex? thePlayer)
        {
            if (thePlayer == null)
            {
                PlayerIndex theReturnedControllingPlayer;
                return IsPressed(theAction, thePlayer, out theReturnedControllingPlayer);
            }

            return IsPressed(theAction, (PlayerIndex)thePlayer);
        }

        public bool IsPressed(string theAction, PlayerIndex? thePlayer, out PlayerIndex theControllingPlayer)
        {
            if (!mInputs.ContainsKey(theAction))
            {
                theControllingPlayer = PlayerIndex.One;
                return false;
            }

            if (thePlayer == null)
            {
                if (IsPressed(theAction, PlayerIndex.One))
                {
                    theControllingPlayer = PlayerIndex.One;
                    return true;
                }

                if (IsPressed(theAction, PlayerIndex.Two))
                {
                    theControllingPlayer = PlayerIndex.Two;
                    return true;
                }

                if (IsPressed(theAction, PlayerIndex.Three))
                {
                    theControllingPlayer = PlayerIndex.Three;
                    return true;
                }

                if (IsPressed(theAction, PlayerIndex.Four))
                {
                    theControllingPlayer = PlayerIndex.Four;
                    return true;
                }

                theControllingPlayer = PlayerIndex.One;
                return false;
            }

            theControllingPlayer = (PlayerIndex)thePlayer;
            return IsPressed(theAction, (PlayerIndex)thePlayer);
        }


        public void AddGamePadInput(string theAction, Buttons theButton, bool isReleasedPreviously)
        {
            MyInput(theAction).AddGamepadInput(theButton, isReleasedPreviously);
        }

        public void AddKeyboardInput(string theAction, Keys theKey, bool isReleasedPreviously)
        {
            MyInput(theAction).AddKeyboardInput(theKey, isReleasedPreviously);
        }

    }
}
