using System;
using System.Diagnostics;
using Microsoft.Xna.Framework;

namespace ErrorReporting
{
#if WINDOWS || XBOX
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main(string[] args)
        {
            Run<Game1>();
        }

        //Used to catch any unhandled exceptions. If the
        //debugger is attached then errors will be shown in the IDE
        //otherwise, if an unhandled exception is encountered a 
        //new exception game is launched giving users a chance
        //to report the error to the developer.
        public static void Run<T>() where T : Game, new()
        {
            if (Debugger.IsAttached)
            {
                using (var game = new T())
                    game.Run();
            }
            else
            {
                try
                {
                    using (var game = new T())
                        game.Run();
                }
                catch (Exception e)
                {
                    using (var game = new ExceptionGame(e))
                        game.Run();
                }
            }
        }
    }
#endif
}

